@@ -1,6 +1,6 @@
 /*
  * Isomorphic SmartClient
- * Version 8.0 (2010-05-18)
+ * Version SC_SNAPSHOT-2010-12-31 (2010-12-31)
  * Copyright(c) 1998 and beyond Isomorphic Software, Inc. All rights reserved.
  * "SmartClient" is a trademark of Isomorphic Software, Inc.
  *
@@ -108,17 +108,23 @@ isc.TextItem.addProperties({
     _dataElementIsTextBox:true,
 
     //> @attr   textItem.emptyStringValue   (any : null : IRW)
-    //      Should the empty string be mapped to null, or stored as an empty string.
-    //      Updated on 'setValue(null)' or 'setValue("")'
+    // How should an empty string entered by the user be stored?
+    // This value is typically set to <code>null</code> or <code>""</code>.
+    // <P>
+    // Note that a call to +link{setValue(),setValue(null)} or +link{setValue(),setValue("")}
+    // automatically updates this property to ensure that "empty" values are stored in a 
+    // consistent format.
     // @group formValues
-    // @visibility   internal
+    // @visibility   external
     //<    
     
-    _emptyStringValue:null,
+    emptyStringValue:null,
     
     // Override redrawOnShowFormIcon - we can handle dynamically updating the item's HTML to
     // show / hide text item icons
     redrawOnShowIcon:false,
+    // setting clipValue to true ensures we resize the text box when showing/hiding icons
+    clipValue:true,
     
     // _nativeEventHandlers is a place to specify native event handlers to be applied to the
     // form item element once it has been written into the DOM (without having to override 
@@ -135,12 +141,28 @@ isc.TextItem.addProperties({
     
             } : null
         )
-    }
+    },
     
     //>@attr TextItem.browserSpellCheck (boolean : null : IRWA)
     // @include FormItem.browserSpellCheck
     // @visibility internal
     //<
+   
+    //>@attr textItem.browserAutoCapitalize (boolean : null : IRWA)
+    // @include FormItem.browserAutoCapitalize
+    // @visibility internal
+    //<
+    
+    //>@attr textItem.browserAutoCorrect (boolean : null : IRWA)
+    // @include formItem.browserAutoCorrect
+    // @visibility internal
+    //<
+   
+    //>@attr texttem.browserInputType (browserInputType : null : IRWA)
+    // @include formItem.browserInputType
+    // @visibility internal
+    //<
+    
     
     //>@attr TextItem.selectOnFocus (boolean : null : IRW)
     // @include FormItem.selectOnFocus
@@ -186,6 +208,17 @@ isc.TextItem.addProperties({
     // @see FormItem.hint
     // @visibility external
     //<
+    
+    //>@attr TextItem.printFullText (boolean : false : IRW)
+    // When generating a print-view of the component containing this TextItem, should
+    // the form item expand to accomodate its value? If set to false the text box will not expand
+    // to fit its content in the print view, instead showing exactly as it does in the
+    // live form.
+    // @visibility external
+    // @group printing
+    //<
+    printFullText:false
+
 
                                         
 });
@@ -251,80 +284,109 @@ isc.TextItem.addMethods({
         var valueIconHTML = this._getValueIconHTML(dataValue);
         if (this.showValueIconOnly) return valueIconHTML;
         
-		var template = this._$elementStartTemplate,
-            form = this.form,
-			formID = form.getID(),
-			itemID = this.getItemID()
-		;
-
-        // May be null
-        template[0] = valueIconHTML;
-		
-        template[2] = this._elementType;
-        template[4] = this.getElementName();
-        template[6] = this.getDataElementId();
+        var result;
         
-        // hang a flag on the element marking it as the data element for the
-        // appropriate form item.
-        template[8] = this._getItemElementAttributeHTML();
-        
-        // At this point we're appending to the end of the template Disable spellchecker in
-        // Moz if appropriate so we don't get the red wavy line under email addresses etc.
+        if (this._isPrinting() && this.printFullText) {
+            result = isc.StringBuffer.concat(
+                "<SPAN ",this.getElementStyleHTML(), ">",
+                dataValue == null ? "&nbsp;" : dataValue.asHTML(), "</SPAN>"
+            );
+        } else {
+            
+            var template = this._$elementStartTemplate,
+                form = this.form,
+                formID = form.getID(),
+                itemID = this.getItemID()
+            ;
+    
+            // May be null
+            template[0] = valueIconHTML;
+            
+            template[2] = this._elementType;
+            template[4] = this.getElementName();
+            template[6] = this.getDataElementId();
+            
+            // hang a flag on the element marking it as the data element for the
+            // appropriate form item.
+            template[8] = this._getItemElementAttributeHTML();
+            
+            // At this point we're appending to the end of the template Disable spellchecker in
+            // Moz if appropriate so we don't get the red wavy line under email addresses etc.
+             
+            
+            if (isc.Browser.isMoz || isc.Browser.isSafari) {
+                if (this.getBrowserSpellCheck()) template[template.length] = " spellcheck=true";
+                else template[template.length] = " spellcheck=false"
+            }
          
+            // iPhone / Safari specific native features
+            if (isc.Browser.isSafari) {
+                if (this.browserAutoCapitalize == false) {
+                    template[template.length] = " autocapitalize=off";
+                }
+                if (this.browserAutoCorrect == false) {
+                    template[template.length] = " autocorrect=off";
+                }
+                if (this.browserInputType != null) {
+                    var nativeType = this.browserInputTypeMap[this.browserInputType];
+                    if (nativeType == null) nativeType = this.browserInputType;
+                    template[template.length] = " type='" + nativeType + "'";
+                }
+            }
+            
+            // If we get an oninput event for this browser, write it out into our element's HTML
+            
+            if (this._willHandleInput) {
+                template[template.length] = " ONINPUT='" 
+                template[template.length] = this.getID() 
+                template[template.length] = "._handleInput()'"
+            }
+            
+            if (this.isDisabled()) template[template.length] = this._$disabled;
+            
+            // Write out 'readOnly' setting if present
+            if (this.isInactiveHTML() || this.readOnly) {
+                template[template.length] = " READONLY=TRUE"
+            }
+            
+            if (this.isInactiveHTML() && value != null && value != isc.emptyString) {
+                // run the value through makeXMLSafe - this'll escape single quotes
+                // / avoid terminating the tag if a '>' char is present etc
+                template[template.length] = " value='" + isc.makeXMLSafe(value) + "'";
+            }
+            
+            // disable native autoComplete 
+                  
+            if (this._getAutoCompleteSetting() != this._$native) {
+                template[template.length] = this._$autoCompleteOff;
+            }
+            
+            template[template.length] = this.getElementStyleHTML();
+            
+            
+            var tabIndex = this._getElementTabIndex();
+            if (tabIndex != null) {
+                var end = template.length;
+                template[end] = this._$tabIndexEquals;  
+                isc._fillNumber(template, tabIndex, end+1, 5);
+            }
+            
+            // Note: if we're showing a title for the element, we don't need to set
+            // up an accessKey here, since the label tag takes care of that
+            if (this.showTitle == false && this.accessKey != null) {
+                template[template.length] = this._$accessKeyEquals;
+                template[template.length] = this.accessKey;
+            }
+            
+            template[template.length] = this._$rightAngle;
+    
+            result = template.join(isc.emptyString);
         
-        if (isc.Browser.isMoz || isc.Browser.isSafari) {
-            if (this.getBrowserSpellCheck()) template[template.length] = " spellcheck=true";
-            else template[template.length] = " spellcheck=false"
+            // Trim the entries off the end of the template so we can reuse it.
+            template.length = 8;
         }
+        //this.logWarn("generated textItem HTML:"+ result);
         
-        // If we get an oninput event for this browser, write it out into our element's HTML
-        
-        if (this._willHandleInput) {
-            template[template.length] = " ONINPUT='" 
-            template[template.length] = this.getID() 
-            template[template.length] = "._handleInput()'"
-        }
-        
-        if (this.isDisabled()) template[template.length] = this._$disabled;
-        
-        // Write out 'readOnly' setting if present
-        if (this.isInactiveHTML() || this.readOnly) {
-            template[template.length] = " READONLY=TRUE"
-        }
-        
-        if (this.isInactiveHTML() && value != null && value != isc.emptyString) {
-            template[template.length] = " value='" + value + "'";
-        }
-        
-        // disable native autoComplete 
-              
-        if (this._getAutoCompleteSetting() != this._$native) {
-            template[template.length] = this._$autoCompleteOff;
-        }
-        
-        template[template.length] = this.getElementStyleHTML();
-        
-        
-        var tabIndex = this._getElementTabIndex();
-        if (tabIndex != null) {
-            var end = template.length;
-            template[end] = this._$tabIndexEquals;  
-            isc._fillNumber(template, tabIndex, end+1, 5);
-        }
-        
-        // Note: if we're showing a title for the element, we don't need to set
-        // up an accessKey here, since the label tag takes care of that
-        if (this.showTitle == false && this.accessKey != null) {
-            template[template.length] = this._$accessKeyEquals;
-            template[template.length] = this.accessKey;
-        }
-        
-        template[template.length] = this._$rightAngle;
-
-        var result = template.join(isc.emptyString);
-        
-        // Trim the entries off the end of the template so we can reuse it.
-        template.length = 8;
         return result;
 	},  
     
@@ -419,8 +481,8 @@ isc.TextItem.addMethods({
             // whether this element can recieve focus or not.
             // (slots 18 and 19)
         (isc.Browser.isMoz ? "-moz-user-focus:" 
-            
-            : isc.Browser.isIE ? "margin-top:-1px;margin-bottom:-1px;" : null),    // [18]
+          
+            :  null),    // [18]
         ,                   // [19] Moz: 'normal' or 'ignore' - otherwise null
         "' "                // [20]
     ],
@@ -430,9 +492,18 @@ isc.TextItem.addMethods({
     _$textAlignColon:"text-align:",
     _$semi:";",
     _$normal:"normal;", _$ignore:"ignore;",
+    
+    _$negativeMargins:"margin-top:-1px;margin-bottom:-1px;",
 	getElementStyleHTML : function () {
         
+	    // in 'printFullText' / printing mode we write out a span rather than
+	    // an input.
+	    // Most of the css will be the same but we can skip a few steps
+	    var isStaticElement = this._isPrinting() && this.printFullText;
+	    
         var template = this._$styleTemplate,
+        
+        
             width = this.getTextBoxWidth(),
             height = this.getTextBoxHeight(),
             style = this.getTextBoxStyle();
@@ -465,14 +536,35 @@ isc.TextItem.addMethods({
         } else {
             template[15] = template[16] = template[17] = null;
         }
-        
-        if (isc.Browser.isMoz) {
+        if (isc.Browser.isIE) {
+            template[18] = isStaticElement ? null : this._$negativeMargins;
+        }
+        if (isc.Browser.isMoz && !isStaticElement) {
             template[19] = (this._getElementTabIndex() > 0 ? this._$normal
                                                            : this._$ignore);
         }
         return template.join(isc.emptyString);
     },
     
+    //> @method textItem.getEnteredValue()
+    // Returns the raw text value typed into this form field, which can differ from 
+    // +link{formItem.getValue()} in various cases - for example:
+    // <ul>
+    // <li>for items that contrain the value range, such as a +link{DateItem} with
+    // +link{DateItem.enforceDate,enforceDate}:true, or a +link{ComboBoxItem} with
+    // +link{ComboBoxItem.addUnkownValues,addUnknownValues}:false</li>
+    // <li>for items with a defined valueMap or edit value formatter and parser functions
+    // which converts display value to data value</li>
+    // <li>while the item has focus if +link{changeOnKeypress} is false
+    // </li></ul>
+    // @return (string) current entered value
+    // @visibility external
+    //<
+    getEnteredValue : function () {
+        return this.getElementValue();
+    },
+    
+    
     //>@method textItem.mapValueToDisplay()  (A)
     // Map from the internal value for this item to the display value.
     // @param   internalValue   (string)   Internal value for this item.
@@ -506,18 +598,17 @@ isc.TextItem.addMethods({
     // @return  (string)   Internal value corresponding to that display value.
     //<
     mapDisplayToValue : function (displayValue) {
+        var value;
        
         if (this.mask) {
-            var value = this._unmaskValue(displayValue);
+            value = this._unmaskValue(displayValue);
         } else {
             value = this._unmapKey(displayValue);
         }
-        if (!this.applyStaticTypeFormat && this.parseEditorValue != null) {
-            return this.parseEditorValue(displayValue, this.form, this);
-        }
+        value = this._parseDisplayValue(value);
 
         // if the value to be saved is an empty string, map it to 'null' if necessary
-        if (isc.is.emptyString(value)) value = this._emptyStringValue;
+        if (isc.is.emptyString(value)) value = this.emptyStringValue;
         return value;
     },
     
@@ -542,7 +633,7 @@ isc.TextItem.addMethods({
 
         var undef;
         if (value !== undef && (value == null || isc.is.emptyString(value)))
-            this._emptyStringValue = value;
+            this.emptyStringValue = value;
 
         // Translate incoming value based on characterCasing if needed
         if (value !== undef && value != null && this.characterCasing != isc.TextItem.DEFAULT) {
@@ -570,7 +661,7 @@ isc.TextItem.addMethods({
     // item name
     getCriteriaFieldName : function () {
         if (this.displayField) return this.displayField;
-        return this.getFieldName();
+        return this.Super("getCriteriaFieldName", arguments);
     },
 
     // When focus is received, the hint should be hidden if TextItem.showHintInField is true.
@@ -713,7 +804,10 @@ isc.TextItem.addMethods({
                         }
                     } else {
                         if (this.maskOverwriteMode) {
-                            this._maskBuffer[pos] = this.maskPromptChar;
+                            // Don't clear a non-entry position
+                            if (pos == this._getNextEntryPosition (pos - 1)) {
+                                this._maskBuffer[pos] = this.maskPromptChar;
+                            }
                         } else {
                             this._shiftMaskBufferLeft (pos);
                         }
@@ -1263,7 +1357,7 @@ isc.TextItem.addMethods({
 
         // Move each character <len> positions to the left where the character
         // matches the new position's filter.
-        for (var i = pos, pos2 = i+len-1; i < this._length; i++) {
+        for (var i = pos, pos2 = i+len-1; i < this._length; i++, pos2 = j) {
             if (this._maskFilters[i]) {
                 this._maskBuffer[i] = this.maskPromptChar;
                 var j = this._getNextEntryPosition (pos2++);
