@@ -24,6 +24,14 @@ public interface BankDetailService extends RemoteService {
 	 */
 	Map<String, String> getDetailedBankInfo(String bankName);
 	
+	/**
+	 * Return dependencies from given bank.
+	 * 
+	 * @param bankName
+	 * @return
+	 */
+	Map<String, String> getDependencies(String bankName);
+	
 	/**
 	 * Returns the field list for a bank organized by sections.
 	 * 
@@ -33,10 +41,10 @@ public interface BankDetailService extends RemoteService {
 	Map<String, List<String>> getFields(boolean restricted);
 	
 	/**
-	 * Returns the types list for all the banks.
+	 * Returns the types list for the banks belonging to given user group.
 	 * 
 	 * @return
 	 */
-	List<String> getTypes();
+	List<String> getTypes(String login);
 
 }
