@@ -13,10 +13,13 @@ import org.inria.bmajwatcher.client.services.BankDetailServiceAsync;
 import org.inria.bmajwatcher.client.services.BankRemovalServiceAsync;
 import org.inria.bmajwatcher.client.services.BankStatServiceAsync;
 import org.inria.bmajwatcher.client.services.BiomajLauncherServiceAsync;
+import org.inria.bmajwatcher.client.services.UserManagerServiceAsync;
 import org.inria.bmajwatcher.client.ui.bank_editor.BankEditorDialog;
+import org.inria.bmajwatcher.client.ui.confirmation.MessagePanel;
 import org.inria.bmajwatcher.client.ui.scheduler.Scheduler;
 import org.inria.bmajwatcher.client.ui.styles.MainCss;
 import org.inria.bmajwatcher.client.ui.styles.Resources;
+import org.inria.bmajwatcher.client.ui.users.UserManagerDialog;
 
 
 import com.google.gwt.core.client.GWT;
@@ -97,6 +100,7 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	
 	interface HomePageUiBinder extends UiBinder<Widget, HomePage> {
 	}
+	
 	private static HomePageUiBinder uiBinder = GWT.create(HomePageUiBinder.class);
 
 	@UiField
@@ -114,6 +118,9 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	private Anchor freeSpaceAnchor = new Anchor("Free space");
 	private Anchor distributionAnchor = new Anchor("Banks by server");
 	private Anchor allTypesAnchor = new Anchor("Banks by type");
+	private Anchor bankSizeByUser = new Anchor("Bank size by user");
+	private Anchor bankCountByUser = new Anchor("Bank count by user");
+	
 	private PieChartGenerator cGen;
 	private HTML total = new HTML();
 	private HTML updating = new HTML();
@@ -126,10 +133,15 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	private ToolStripButton editBank;
 	private ToolStripButton showSched;
 	
+	private String loggedUser = null;
+	private boolean isAdmin = false;
+	
 	private ListGrid grid;
 	private MainCss css = Resources.INSTANCE.mainCss();
 	
 	private ToolStripMenuButton updateMenu = null;
+	
+	private UserManagerDialog diag;
 
 	public HomePage() {
 		initWidget(uiBinder.createAndBindUi(this));
@@ -139,7 +151,6 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		contentPanel.setStylePrimaryName(css.center());
 		rightPanel.setStylePrimaryName(css.right());
 		
-		
 		init();
 		
 		/*
@@ -151,51 +162,66 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	
 	/**
 	 * Inits the components.
-	 * This method is called when the interface has to be redrawn after admin login/logout.
+	 * This method is called when the interface has to be redrawn after user login/logout.
 	 */
 	private void init() {
-		
-		AuthenticationServiceAsync.INSTANCE.getUserAcces(new AsyncCallback<String>() {			
+
+		AuthenticationServiceAsync.INSTANCE.getUserAccess(new AsyncCallback<String>() {			
 			
 			@Override
 			public void onSuccess(String result) {
-				if (result != null) { // Admin is logged
+				if (result != null) { // Some user is logged
+					String[] split = result.split(":");
+					loggedUser = split[0];
+					isAdmin = split[1].equals("admin");
+					UserManagerServiceAsync.INSTANCE.getUserKey(loggedUser, new AsyncCallback<String>() {
+						
+						@Override
+						public void onSuccess(String userKey) {
+							drawContext(true, userKey);
+						}
+						
+						@Override
+						public void onFailure(Throwable caught) {
+							SC.warn("RPC failure");
+						}
+					});
 					
-					drawContext(true);
 					
 				} else {
-					
-					drawContext(false);
+					isAdmin = false;
+					loggedUser = "";
+					drawContext(false, null);
 					
 				}
 			}
 			
 			@Override
 			public void onFailure(Throwable caught) {
-				SC.warn("User access failure");
+				SC.warn("User access : RPC failure");
 			}
 		});
 	}
 	
 	/**
-	 * Draws the components according to the user group (admin or guest).
+	 * Draws the components according to the user group (logged or guest).
 	 * 
-	 * @param adminMode
+	 * @param logged
 	 */
-	private void drawContext(final boolean adminMode) {
+	private void drawContext(final boolean logged, final String userKey) {
 		
 		DecoratedTabPanel tabPanel = new DecoratedTabPanel();
 		final VLayout bankLayout = new VLayout();
 		bankLayout.setMembersMargin(5);
 		final VLayout errorLayout = new VLayout();
-		final ListGrid errorGrid = getErrorGrid();
+		final ListGrid errorGrid = getErrorGrid(userKey);
 		
 		tabPanel.addSelectionHandler(new SelectionHandler<Integer>() {
 			
 			@Override
 			public void onSelection(SelectionEvent<Integer> event) {
 				if (event.getSelectedItem() == 1) {
-					BankStatServiceAsync.INSTANCE.getBanksInfo(new AsyncCallback<List<Map<String,String>>>() {
+					BankStatServiceAsync.INSTANCE.getBanksInfo(userKey, new AsyncCallback<List<Map<String,String>>>() {
 						
 						@Override
 						public void onSuccess(List<Map<String, String>> result) {
@@ -207,7 +233,7 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 							SC.warn("Failure");
 						}
 					});
-				} else if (adminMode && event.getSelectedItem() == 2) {
+				} else if (logged && event.getSelectedItem() == 2) {
 					// Hide the layout that contains the bank list otherwise
 					// conflicts with the layout that contains the error grid causing
 					// bad handling of mouse clicks on the component.
@@ -215,7 +241,7 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 					errorLayout.show();
 					errorGrid.invalidateCache();
 					errorGrid.fetchData();
-				} else if (adminMode && event.getSelectedItem() == 0) {
+				} else if (logged && event.getSelectedItem() == 0) {
 					errorLayout.hide();
 					bankLayout.show();
 				}
@@ -230,13 +256,14 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		globalStatsMenu = new VerticalPanel();
 		globalStatsMenu.setStylePrimaryName(css.graphMenu());
 		globalGraphPanel = new VerticalPanel();
+		globalGraphPanel.setHeight("543px");
 		globalStatsPanel.add(globalStatsMenu);
 		globalStatsPanel.add(globalGraphPanel);
 		
 		/*
 		 * BANK LIST
 		 */
-		grid = getBankGrid(adminMode);
+		grid = getBankGrid(logged, userKey);
 		
 		/*
 		 * REFRESH BUTTON
@@ -249,56 +276,79 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			@Override
 			public void onClick(com.google.gwt.event.dom.client.ClickEvent event) {
 				grid.invalidateCache();
-				updateStatusBar(adminMode);
+				grid.fetchData(new Criteria(), new DSCallback() {
+					
+					@Override
+					public void execute(DSResponse response, Object rawData, DSRequest request) {
+						updateStatusBar(logged);
+					}
+				});
 			}
 		});
 		
 
-		bankLayout.addMember(getToolBar(adminMode));
+		bankLayout.addMember(getToolBar(logged, userKey));
 		bankLayout.addMember(grid);
 		bankLayout.setWidth(684);
-		bankLayout.setHeight(500);
+		bankLayout.setHeight(513);
 		
 		rightPanel.clear();
 		/*
 		 * LOGIN PANEL
 		 */
-		rightPanel.add(getLoginPanel(adminMode));
-		/*
-		 * SCHEDULER
-		 */
-		if (adminMode)
-			rightPanel.add(getScheduler(grid));
+		rightPanel.add(getLoginPanel(logged));
 
 		VerticalPanel pnl = new VerticalPanel();
 		pnl.add(bankLayout);
 		HorizontalPanel hPnl = new HorizontalPanel();
 		hPnl.add(refreshGrid);
+		
+		if (logged) {
+			/*
+			 * Show only user banks
+			 */
+			final CheckBox showGroupOnly = new CheckBox("Show group banks only");
+			showGroupOnly.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
+				
+				@Override
+				public void onClick(ClickEvent event) {
+					if (showGroupOnly.getValue()) {
+						grid.filterData(new Criteria(BankGridDataSource.GROUP, "true"));
+					} else {
+						grid.clearCriteria();
+						grid.filterData();
+					}
+				}
+			});
+			hPnl.add(showGroupOnly);
+			hPnl.setCellWidth(showGroupOnly, "100%");
+			hPnl.setCellHorizontalAlignment(showGroupOnly, HasHorizontalAlignment.ALIGN_RIGHT);
+		}
+		
 		pnl.add(hPnl);
 		tabPanel.setStylePrimaryName(css.tabPane());
 		tabPanel.add(pnl, BmajWatcherConstants.INSTANCE.banksStatus());
 		tabPanel.add(globalStatsPanel, BmajWatcherConstants.INSTANCE.statistics());
-		if (adminMode) {
+		if (logged) {
 			/*
 			 * ERROR PANEL
 			 */
 			VerticalPanel errorsPanel = new VerticalPanel();
 			errorLayout.addMember(errorGrid);
 			errorLayout.setWidth(684);
-			errorLayout.setHeight(480);
+			errorLayout.setHeight(543);
 			errorsPanel.add(errorLayout);
 			tabPanel.add(errorsPanel, BmajWatcherConstants.INSTANCE.errors());
 		}
 		
 		tabPanel.selectTab(0);
 		
-		
 		contentPanel.clear();
 		contentPanel.add(tabPanel);
-		contentPanel.add(getStatusBar(adminMode));
+		contentPanel.add(getStatusBar(logged));
 	}
 	
-	private ToolStrip getToolBar(final boolean adminMode) {
+	private ToolStrip getToolBar(final boolean logged, final String key) {
 		ToolStrip ts = new ToolStrip();
 		ts.setWidth100();
 		
@@ -327,11 +377,11 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			
 			@Override
 			public void onClick(com.smartgwt.client.widgets.events.ClickEvent event) {
-				showBankDetail(selectedRecord, adminMode);
+				showBankDetail(selectedRecord, logged && Boolean.valueOf(selectedRecord.getAttributeAsString(BankGridDataSource.GROUP)), key);
 			}
 		});
 		
-		if (adminMode) {
+		if (logged) {
 			Menu updateItems = new Menu();
 			MenuItem itemNormalUpdate = new MenuItem("biomaj.sh --update", "normal-update.png");;
 			MenuItem itemUpdateFromScratch = new MenuItem("biomaj.sh --update --fromscratch", "update-from-scratch.png");
@@ -413,12 +463,12 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 				@Override
 				public void onClick(com.smartgwt.client.widgets.events.ClickEvent event) {
 					if (selectedRecord != null)
-						BankEditorDialog.getInstance().show(selectedRecord.getAttribute(BankGridDataSource.NAME));
+						BankEditorDialog.getInstance(loggedUser).show(selectedRecord.getAttribute(BankGridDataSource.NAME));
 				}
 			});
 			ts.addButton(editBank);
 			
-			showSched.setIcon("show-in-sched.png");
+			showSched.setIcon("clock-go.png");
 			showSched.setTitle("Show task");
 			showSched.setDisabled(selectedRecord == null);
 			showSched.addClickHandler(new com.smartgwt.client.widgets.events.ClickHandler() {
@@ -435,13 +485,13 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			ts.addSeparator();
 			
 			ToolStripButton bankEditor = new ToolStripButton();
-			bankEditor.setIcon("show-bank-editor.png");
+			bankEditor.setIcon("page-edit.png");
 			bankEditor.setTitle("New bank");
 			bankEditor.addClickHandler(new com.smartgwt.client.widgets.events.ClickHandler() {
 				
 				@Override
 				public void onClick(com.smartgwt.client.widgets.events.ClickEvent event) {
-					BankEditorDialog.getInstance().show();
+					BankEditorDialog.getInstance(loggedUser).show();
 				}
 			});
 			ts.addButton(bankEditor);
@@ -473,6 +523,12 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		allTypesAnchor.addClickHandler(this);
 		topMenu.add(distributionAnchor);
 		distributionAnchor.addClickHandler(this);
+		if (isAdmin) {
+			topMenu.add(bankSizeByUser);
+			bankSizeByUser.addClickHandler(this);
+			topMenu.add(bankCountByUser);
+			bankCountByUser.addClickHandler(this);
+		}
 		
 		topMenu.add(new HTML("<b>Types</b>"));
 		
@@ -491,7 +547,7 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		globalGraphPanel.add(cGen.getAllBanks());
 	}
 	
-	private ListGrid getErrorGrid() {
+	private ListGrid getErrorGrid(String userKey) {
 		ListGrid errors = new ListGrid() {
 			@Override
 			protected Canvas getExpansionComponent(ListGridRecord record) {
@@ -530,7 +586,7 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		errors.setFields(typeField, bankField, logField, sourceField, dateField);
 		
 
-		errors.setDataSource(ErrorGridDataSource.getInstance());
+		errors.setDataSource(ErrorGridDataSource.getInstance(userKey));
 		errors.setAutoFetchData(false);
 		
 		return errors;
@@ -539,28 +595,45 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	/**
 	 * Generates the grid that contains the bank list.
 	 * 
-	 * @param adminMode whether some elements should be displayed or not
+	 * @param logged whether some elements should be displayed or not
 	 * @return
 	 */
-	private ListGrid getBankGrid(final boolean adminMode) {
+	private ListGrid getBankGrid(final boolean logged, final String key) {
 		
 		final ListGrid banks = new ListGrid() {
 			@Override
 			protected String getCellCSSText(ListGridRecord record, int rowNum, int colNum) {
+				String style = "";
+				
+				// Change row color for user banks
+				if (Boolean.valueOf(record.getAttributeAsString(BankGridDataSource.OWNER)))
+//					style = "background-color:#E9E6FE;";
+					style = "background-color:#EDEDFE;";
+				else if (Boolean.valueOf(record.getAttributeAsString(BankGridDataSource.GROUP)))
+//					style = "background-color:#EDEDFE;";
+					style = "background-color:#EBDBFE;";
+				else if (!isAdmin) // Can't drag records if not owner
+					record.setCanDrag(false);
+				
 				if (getFieldName(colNum).equals(BankGridDataSource.STATUS)) {
+					
 					if (record.getAttributeAsString(BankGridDataSource.STATUS).equalsIgnoreCase("OK")) {
-						return "font-weight:bold; color:GREEN;";
+						style += "font-weight:bold; color:GREEN;";
 					} else if (record.getAttributeAsString(BankGridDataSource.STATUS).equalsIgnoreCase("KO")) {
-						return "font-weight:bold; color:RED;";
+						style += "font-weight:bold; color:RED;";
 					} else if (record.getAttributeAsString(BankGridDataSource.STATUS).equalsIgnoreCase("updating")) {
-						return "font-weight:bold; color:BLACK;";
+						style += "font-weight:bold; color:BLACK;";
 					} else
-						return super.getCellCSSText(record, rowNum, colNum);
+						style += super.getCellCSSText(record, rowNum, colNum);
 				} else
-					return super.getCellCSSText(record, rowNum, colNum);
+					style += super.getCellCSSText(record, rowNum, colNum);
+				
+				return style;
 			}
 		};
 		
+		banks.setBaseStyle("bankGridCell");
+		
 		banks.setShowFilterEditor(true);
 		banks.setShowAllRecords(false);
 		banks.setWidth100();
@@ -577,19 +650,26 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 					com.smartgwt.client.widgets.grid.events.SelectionEvent event) {
 				
 				selectedRecord = event.getRecord();
-				
-				showSched.setDisabled(false);
-				editBank.setDisabled(false);
 				detailIcon.setDisabled(false);
 				
-				boolean isUpdating = selectedRecord.getAttribute(BankGridDataSource.STATUS).equalsIgnoreCase("updating");
-				if (updateMenu != null) {
-//					String imgHtml = Canvas.imgHTML("system-switch-user.png", 16, 16);
-					String imgHtml = "";
-					updateMenu.setTitle(imgHtml + " <span class=\"" + css.toolBarMenuSmall() + "\">Update " + selectedRecord.getAttribute(BankGridDataSource.NAME) + "</span>");
-					updateMenu.setDisabled(isUpdating);
+				if (logged && (isAdmin || Boolean.valueOf(selectedRecord.getAttributeAsString(BankGridDataSource.GROUP)))) {
+					showSched.setDisabled(false);
+					editBank.setDisabled(false);
+					
+					boolean isUpdating = selectedRecord.getAttribute(BankGridDataSource.STATUS).equalsIgnoreCase("updating");
+					if (updateMenu != null) {
+						String imgHtml = "";
+						updateMenu.setTitle(imgHtml + " <span class=\"" + css.toolBarMenuSmall() + "\">Update " + selectedRecord.getAttribute(BankGridDataSource.NAME) + "</span>");
+						updateMenu.setDisabled(isUpdating);
+					}
+					removeBank.setDisabled(isUpdating);
+				} else {
+					showSched.setDisabled(true);
+					editBank.setDisabled(true);
+					if (updateMenu != null)
+						updateMenu.setDisabled(true);
+					removeBank.setDisabled(true);
 				}
-				removeBank.setDisabled(isUpdating);
 			}
 		});
 		
@@ -598,7 +678,7 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			@Override
 			public void onRecordDoubleClick(RecordDoubleClickEvent event) {
 				Record record = event.getRecord();
-				showBankDetail(record, adminMode);
+				showBankDetail(record, logged && (isAdmin || Boolean.valueOf(record.getAttributeAsString(BankGridDataSource.GROUP))), key);
 			}
 		});
 		
@@ -607,6 +687,13 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		schedField.setAlign(Alignment.CENTER);
 		schedField.setType(ListGridFieldType.IMAGE);
 		schedField.setCanSort(false);
+		
+		ListGridField visibilityField = new ListGridField(BankGridDataSource.VISIBILITY, "Access", 60);
+		visibilityField.setAlign(Alignment.CENTER);
+		visibilityField.setType(ListGridFieldType.IMAGE);
+		visibilityField.setImageURLSuffix(".png");
+		visibilityField.setCanSort(false);
+		
 		ListGridField typeField = new ListGridField(BankGridDataSource.TYPE, BmajWatcherConstants.INSTANCE.type());
 		ListGridField formatsField = new ListGridField(BankGridDataSource.FORMATS, BmajWatcherConstants.INSTANCE.formats());
 		ListGridField nameField = new ListGridField(BankGridDataSource.NAME, BmajWatcherConstants.INSTANCE.name());
@@ -615,11 +702,11 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		
 		ListGridField statusField = new ListGridField(BankGridDataSource.STATUS, BmajWatcherConstants.INSTANCE.status(), 60);
 		
-		if (adminMode) {
+		if (logged) {
 			banks.setCanDragRecordsOut(true);
 			banks.setDragDataAction(DragDataAction.COPY);
 			
-			banks.setFields(schedField, typeField, formatsField, nameField, releaseField, sessionField, statusField);
+			banks.setFields(schedField, typeField, formatsField, nameField, releaseField, sessionField, statusField, visibilityField);
 			
 			banks.addCellContextClickHandler(new CellContextClickHandler() {
 				
@@ -627,21 +714,25 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 				public void onCellContextClick(CellContextClickEvent event) {
 					event.cancel();
 					Record record = event.getRecord();
-					showPopupMenu(record, banks);
+					// Popup for group only
+					if (logged && (isAdmin || Boolean.valueOf(record.getAttributeAsString(BankGridDataSource.GROUP))))
+						showPopupMenu(record, banks);
 				}
 			});
 		}
 		else
 			banks.setFields(typeField, formatsField, nameField, releaseField, sessionField);
 		
-		
-		banks.setDataSource(BankGridDataSource.getInstance());
 		banks.setAutoFetchData(false);
-		banks.fetchData(banks.getCriteria(), new DSCallback() {
+		banks.setDataSource(BankGridDataSource.getInstance(key));
+		
+		banks.fetchData(new Criteria(), new DSCallback() {
 			
 			@Override
 			public void execute(DSResponse response, Object rawData, DSRequest request) {
-				updateStatusBar(adminMode);
+				updateStatusBar(logged);
+				if (logged)
+					rightPanel.add(getScheduler(grid, key));
 			}
 		});
 
@@ -655,15 +746,7 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	 * @return
 	 */
 	private Composite getLoginPanel(final boolean logged) {
-		
-		HorizontalPanel namePanel = new HorizontalPanel();
-		HorizontalPanel passwdPanel = new HorizontalPanel();
 		HorizontalPanel okPanel = new HorizontalPanel();
-		
-		VerticalPanel container = new VerticalPanel();
-		container.add(namePanel);
-		container.add(passwdPanel);
-		container.add(okPanel);
 				
 		Label login = new Label(BmajWatcherConstants.INSTANCE.userName());
 		login.setStylePrimaryName(css.formLabel());
@@ -676,15 +759,37 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		
 		Button submitButton;
 		
+		VerticalPanel container = new VerticalPanel();
+		HorizontalPanel namePanel = new HorizontalPanel();
+		container.add(namePanel);
+		
 		if (!logged) {
+			HorizontalPanel passwdPanel = new HorizontalPanel();
+			
 			namePanel.add(login);
 			namePanel.add(loginTxt);
 			
 			passwdPanel.add(passwd);
 			passwdPanel.add(passwdTxt);
 			
+			container.add(passwdPanel);
+			
 			submitButton = new Button(BmajWatcherConstants.INSTANCE.submit());
 		} else {
+			namePanel.add(new HTML("<b>Welcome</b>&nbsp;"));
+			Anchor ancLogin = new Anchor(loggedUser);
+			ancLogin.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
+				
+				@Override
+				public void onClick(ClickEvent event) {
+					diag = new UserManagerDialog(loggedUser);
+					diag.addStyleName(css.bankEditor());
+					diag.setPopupPosition(100, 100);
+					diag.show();
+				}
+			});
+			namePanel.add(ancLogin);
+			
 			submitButton = new Button(BmajWatcherConstants.INSTANCE.logout());
 		}
 		
@@ -693,19 +798,18 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		 * CATCH ENTER
 		 */
 		loginTxt.addKeyPressHandler(new KeyPressHandler() {
-			
 			@Override
 			public void onKeyPress(KeyPressEvent event) {
-				if (event.getCharCode() == KeyCodes.KEY_ENTER)
-					requestAuthentication(loginTxt.getValue(), passwdTxt.getValue());;
+				if (event.getNativeEvent().getKeyCode() == KeyCodes.KEY_ENTER)
+					requestAuthentication(loginTxt.getValue(), passwdTxt.getValue());
 			}
 		});
 		passwdTxt.addKeyPressHandler(new KeyPressHandler() {
 			
 			@Override
 			public void onKeyPress(KeyPressEvent event) {
-				if (event.getCharCode() == KeyCodes.KEY_ENTER)
-					requestAuthentication(loginTxt.getValue(), passwdTxt.getValue());;
+				if (event.getNativeEvent().getKeyCode() == KeyCodes.KEY_ENTER)
+					requestAuthentication(loginTxt.getValue(), passwdTxt.getValue());
 			}
 		});
 		
@@ -720,8 +824,9 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 					requestLogout();
 			}
 		});
-		
+
 		submitButton.addStyleName(css.formBtn());
+		container.add(okPanel);
 		okPanel.add(submitButton);
 		
 		DecoratedTabPanel pnl = new DecoratedTabPanel();
@@ -735,50 +840,38 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	/**
 	 * Generates status bar that contains global stats about online banks. 
 	 * 
-	 * @param adminMode
+	 * @param logged
 	 * @param listGridRecords 
 	 * @return
 	 */
-	private Panel getStatusBar(final boolean adminMode) {
+	private Panel getStatusBar(final boolean logged) {
 		
 		VerticalPanel statusBar = new VerticalPanel();
 		statusBar.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
 		statusBar.setStylePrimaryName(css.toolStrip());
 		final HorizontalPanel hp = new HorizontalPanel();
 		hp.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);
-		if (adminMode)
+		if (logged)
 			hp.setWidth("300px");
 		else
 			hp.setWidth("200px");
 		statusBar.add(hp);
 		
-//		BankStatServiceAsync.INSTANCE.banksStatus(new AsyncCallback<Map<String,Integer>>() {
-//			
-//			@Override
-//			public void onSuccess(Map<String, Integer> result) {
-//				online.setHTML("Online : " + result.get("online"));
 		total.setHTML("Total : ?");
 		hp.add(total);
-		if (adminMode) {
+		if (logged) {
 			updating.setHTML("Updating : ?");
 			hp.add(updating);
 			error.setHTML("Error : ?");
 			hp.add(error);
 		}
-//			}
-			
-//			@Override
-//			public void onFailure(Throwable caught) {
-//				SC.warn("Status bar loading failure");
-//			}
-//		});
 		
 		return statusBar;
 		
 	}
 	
 	
-	private void updateStatusBar(final boolean adminMode) {
+	private void updateStatusBar(final boolean logged) {
 		
 		BankStatServiceAsync.INSTANCE.banksStatus(new AsyncCallback<Map<String,Integer>>() {
 			
@@ -801,10 +894,10 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	 * Window that displays detailed info about a bank.
 	 * 
 	 * @param bank
-	 * @param adminMode
+	 * @param logged
 	 */
-	private void showBankDetail(final Record bank, final boolean adminMode) {
-		BankDetailDialog dialog = new BankDetailDialog(bank, adminMode);
+	private void showBankDetail(final Record bank, final boolean logged, String userKey) {
+		BankDetailDialog dialog = new BankDetailDialog(bank, logged, userKey);
 		dialog.show();
 	}
 	
@@ -818,6 +911,12 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 		Menu menu = new Menu();
 		final String bankName = bank.getAttribute(BankGridDataSource.NAME);
 		
+		/*
+		 * Change bank owner
+		 */
+		MenuItem itemChangeOwner = new MenuItem("Change bank owner", "group-go.png");
+		itemChangeOwner.setSubmenu(getUserSubMenu(bankName));
+		
 		/*
 		 * Edit bank properties
 		 */
@@ -826,14 +925,14 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			
 			@Override
 			public void onClick(MenuItemClickEvent event) {
-				BankEditorDialog.getInstance().show(bank.getAttribute(BankGridDataSource.NAME));
+				BankEditorDialog.getInstance(loggedUser).show(bank.getAttribute(BankGridDataSource.NAME));
 			}
 		});
 		
 		/*
 		 * Show scheduled bank in tasks
 		 */
-		MenuItem itemShowTask = new MenuItem("Show task", "show-in-sched.png");
+		MenuItem itemShowTask = new MenuItem("Show task", "clock-go.png");
 		itemShowTask.addClickHandler(new ClickHandler() {
 			
 			@Override
@@ -926,6 +1025,35 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			}
 		});
 		
+		/*
+		 * Change bank visibility
+		 */
+		MenuItem itemVisibility = new MenuItem("Change visibility", "lock-edit.png");
+		itemVisibility.addClickHandler(new ClickHandler() {
+			
+			@Override
+			public void onClick(MenuItemClickEvent event) {
+				final boolean vis = bank.getAttribute(BankGridDataSource.VISIBILITY).equals("public");
+				UserManagerServiceAsync.INSTANCE.changeBankVisibility(bank.getAttribute(BankGridDataSource.NAME),
+						!vis, new AsyncCallback<Boolean>() {
+							
+							@Override
+							public void onSuccess(Boolean result) {
+								if (!result) {
+									SC.warn("Bank visibility could not be changed");
+								} else {
+									MessagePanel.showMessage("Access to bank set to " + (vis ? "private" : "public"));
+								}
+							}
+							
+							@Override
+							public void onFailure(Throwable caught) {
+								SC.warn("RPC failure");
+							}
+						});
+			}
+		});
+		
 		if (!bank.getAttribute(BankGridDataSource.STATUS).equalsIgnoreCase("updating")) {
 			itemFromScratch.setEnabled(true);
 			itemLaunch.setEnabled(true);
@@ -943,12 +1071,57 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			itemStop.setEnabled(false);
 		}
 		
-//		menu.setItems(itemEdit, new MenuItemSeparator(), itemShowTask, new MenuItemSeparator(), itemLaunch, itemFromScratch, itemStop, itemRemove);
-		menu.setItems(itemEdit, new MenuItemSeparator(), itemShowTask, new MenuItemSeparator(), itemLaunch, itemFromScratch, itemNew, itemRebuild, new MenuItemSeparator(), itemRemove);
+		if (isAdmin)
+			menu.setItems(itemVisibility, itemChangeOwner, new MenuItemSeparator(), itemEdit, new MenuItemSeparator(), itemShowTask, new MenuItemSeparator(), itemLaunch, itemFromScratch, itemNew, itemRebuild, new MenuItemSeparator(), itemRemove);
+		else
+			menu.setItems(itemVisibility, new MenuItemSeparator(), itemEdit, new MenuItemSeparator(), itemShowTask, new MenuItemSeparator(), itemLaunch, itemFromScratch, itemNew, itemRebuild, new MenuItemSeparator(), itemRemove);
 		
 		menu.showContextMenu();
 	}
 	
+	private Menu getUserSubMenu(final String bankName) {
+		final Menu userList = new Menu();
+		UserManagerServiceAsync.INSTANCE.getLogins(new AsyncCallback<List<String>>() {
+			
+			@Override
+			public void onSuccess(List<String> result) {
+				for (final String login : result) {
+					MenuItem user = new MenuItem(login);
+					user.addClickHandler(new ClickHandler() {
+						
+						@Override
+						public void onClick(MenuItemClickEvent event) {
+							UserManagerServiceAsync.INSTANCE.changeBankOwner(bankName, login, new AsyncCallback<Boolean>() {
+								
+								@Override
+								public void onSuccess(Boolean result) {
+									if (!result) {
+										SC.warn("Could not change bank owner");
+									} else {
+										MessagePanel.showMessage("Owner changed");
+									}
+								}
+								
+								@Override
+								public void onFailure(Throwable caught) {
+									SC.warn("RPC failure");
+								}
+							});
+						}
+					});
+					userList.addItem(user);
+				}
+			}
+			
+			@Override
+			public void onFailure(Throwable caught) {
+				SC.warn("Could not retrieve user logins");
+			}
+		});
+		
+		return userList;
+	}
+	
 	/**
 	 * Displays dialog that allows user to remove a bank.
 	 * 
@@ -992,6 +1165,16 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			}
 		});
 		
+		final RadioButton rbAllNoHistory = new RadioButton("directories", "All with no history");
+		rbAllNoHistory.addClickHandler(new com.google.gwt.event.dom.client.ClickHandler() {
+			
+			@Override
+			public void onClick(ClickEvent event) {
+				for (CheckBox cb: cbs)
+					cb.setVisible(false);
+			}
+		});
+		
 		pnl.add(rbSelect);
 		canvas.addChild(pnl);
 		final Button btnDelete = new Button("Delete", new com.google.gwt.event.dom.client.ClickHandler() {
@@ -1008,7 +1191,8 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 						BankRemovalServiceAsync.INSTANCE.deleteDirectories(paths, bankName, keepProd.getValue(), new AsyncCallback<Void>() {
 							
 							@Override
-							public void onSuccess(Void result) {	
+							public void onSuccess(Void result) {
+								MessagePanel.showMessage("Remove started");
 							}
 							
 							@Override
@@ -1017,12 +1201,25 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 							}
 						});
 					}
-				} else { // Remove all
-					BankRemovalServiceAsync.INSTANCE.deleteBank(bankName, keepProd.getValue(), new AsyncCallback<Void>() {
+				} else if (rbAll.getValue()) { // Remove all
+					BankRemovalServiceAsync.INSTANCE.deleteBank(bankName, false, keepProd.getValue(), new AsyncCallback<Void>() {
 						
 						@Override
 						public void onSuccess(Void result) {
-							
+							MessagePanel.showMessage("Remove started");
+						}
+						
+						@Override
+						public void onFailure(Throwable caught) {
+							SC.warn("Failure");
+						}
+					});
+				} else { // Remove all no history
+					BankRemovalServiceAsync.INSTANCE.deleteBank(bankName, true, keepProd.getValue(), new AsyncCallback<Void>() {
+						
+						@Override
+						public void onSuccess(Void result) {
+							MessagePanel.showMessage("Remove started");
 						}
 						
 						@Override
@@ -1055,15 +1252,16 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 					pnl.add(cb);
 				}
 				pnl.add(rbAll);
+				pnl.add(rbAllNoHistory);
 				pnl.add(keepProd);
 				HorizontalPanel btnPanel = new HorizontalPanel();
-				btnPanel.setSpacing(5);
-				btnPanel.add(btnCancel);
+				btnPanel.setSpacing(3);
 				btnPanel.add(btnDelete);
+				btnPanel.add(btnCancel);
 //				pnl.add(btnDelete);
 				pnl.add(btnPanel);
 				pnl.setCellHorizontalAlignment(btnPanel, HasHorizontalAlignment.ALIGN_CENTER);
-				winModal.setHeight(150 + result.size() * 40);
+				winModal.setHeight(175 + result.size() * 40);
 			}
 			
 			@Override
@@ -1100,7 +1298,7 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	 * @param login username to be checked
 	 * @param passwd passwd to be checked
 	 */
-	private void requestAuthentication(String login, String passwd) {
+	private void requestAuthentication(final String login, String passwd) {
 		AuthenticationServiceAsync.INSTANCE.authenticate(login, passwd, new AsyncCallback<Boolean>() {
 			
 			@Override
@@ -1120,18 +1318,18 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	}
 	
 	/**
-	 * Returns component for scheduled task handling.
+	 * Returns component for scheduled task handling for the given user.
 	 * 
 	 * @return
 	 */
-	private Composite getScheduler(ListGrid grid) {
+	private Composite getScheduler(ListGrid grid, String userKey) {
 		
-		sched = new Scheduler(grid);
+		sched = new Scheduler(grid, userKey);
 		DecoratedTabPanel tb = new DecoratedTabPanel();
 		tb.setStylePrimaryName(css.scheduler());
 		VerticalPanel pnl = new VerticalPanel();
 		pnl.add(sched.getButtonPanel());
-		pnl.add(sched.getLayout());
+		pnl.add(sched.getContainer());
 		tb.add(pnl, "Scheduler");
 		tb.selectTab(0);
 		
@@ -1139,7 +1337,7 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 	}
 	
 	private void showTypesDialog() {
-		BankDetailServiceAsync.INSTANCE.getTypes(new AsyncCallback<List<String>>() {
+		BankDetailServiceAsync.INSTANCE.getTypes(loggedUser, new AsyncCallback<List<String>>() {
 			
 			@Override
 			public void onSuccess(List<String> result) {
@@ -1261,6 +1459,10 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			globalGraphPanel.add(cGen.getDistribution());
 		} else if (event.getSource() == freeSpaceAnchor){
 			globalGraphPanel.add(cGen.getFreeSpace());
+		} else if (event.getSource() == bankCountByUser) {
+			globalGraphPanel.add(cGen.getBankCountByUser());
+		} else if (event.getSource() == bankSizeByUser) {
+			globalGraphPanel.add(cGen.getSizeByUser());
 		} else {
 			Anchor anc = (Anchor) event.getSource();
 			String type = anc.getText();
@@ -1273,8 +1475,11 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			
 			@Override
 			public void onSuccess(Boolean result) {
-				if (!result)
+				if (!result) {
 					SC.warn("Update failure");
+				} else {
+					MessagePanel.showMessage("Update started");
+				}
 			}
 			
 			@Override
@@ -1289,8 +1494,11 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			
 			@Override
 			public void onSuccess(Boolean result) {
-				if (!result)
+				if (!result) {
 					SC.warn("Update failure");
+				} else {
+					MessagePanel.showMessage("Rebuild started");
+				}
 			}
 			
 			@Override
@@ -1305,8 +1513,11 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 					
 			@Override
 			public void onSuccess(Boolean result) {
-				if (!result)
+				if (!result) {
 					SC.warn("Update failure");
+				} else {
+					MessagePanel.showMessage("Update started");
+				}
 			}
 			
 			@Override
@@ -1321,8 +1532,11 @@ public class HomePage extends Composite implements com.google.gwt.event.dom.clie
 			
 			@Override
 			public void onSuccess(Boolean result) {
-				if (!result)
+				if (!result) {
 					SC.warn("Update failure");
+				} else {
+					MessagePanel.showMessage("Update started");
+				}
 			}
 			
 			@Override
