@@ -74,11 +74,11 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 	}
 	
 	@Override
-	public Map<String, String> getBankProperties(String bankName, String type, Collection<String> included) {
+	public Map<String, String> getBankProperties(String login, String bankName, String type, Collection<String> included) {
 		
 		Map<String, String> res = new HashMap<String, String>();
 		if (type.equals(TYPE_LOCAL)) {
-			return getBankProperties(bankName, included);
+			return getBankProperties(login, bankName, included);
 		} else if (type.equals(TYPE_REMOTE)) {
 			Properties bankOnlyProperties = remoteRetriever.getBankOnlyProperties(bankName);
 			Properties allProps = remoteRetriever.getAllBankProperties(bankName);
@@ -96,38 +96,61 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 	}
 	
 	@Override
-	public Collection<String> getIncludePropertiesList() {
+	public Collection<String> getIncludePropertiesList(String user) {
+		List<String> users = null;
+		if (user.equals(UserManagerServiceImpl.adminLogin()))
+			users = UserManagerServiceImpl.getUserList();
+		else
+			users = UserManagerServiceImpl.getGroupUserFromLogin(user);
+		
+		Set<String> res = new TreeSet<String>();
 		try {
-			File dir = new File(BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR) + "/include");
-			if (dir.exists() && dir.isDirectory()) {
-				Set<String> res = new TreeSet<String>();
+			String root = BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR);
+			for (String u : users) {
+				File dir = new File(root + "/" + u + "/include");
+				if (!dir.exists() || !dir.isDirectory()) {
+					dir.mkdir();
+				}
 				for (File file : dir.listFiles()) {
 					if (file.getName().endsWith(".properties") && !file.isDirectory()) {
 						res.add(file.getName());
 					}
 				}
-				return res;
 			}
-			
 		} catch (BiomajException e) {
-			e.printStackTrace();
+			log.error(e);
 		}
-		return null;
+		
+		return res;
 	}
 
 	@Override
-	public Collection<String> getBankList() {
+	public Collection<String> getBankList(String login) {
+		
+		List<String> users = null;
+		Set<String> res = new TreeSet<String>();
+		if (UserManagerServiceImpl.adminLogin().equals(login)) {
+			users = UserManagerServiceImpl.getUserList();
+			res.add("global");
+		}
+		else
+			users = UserManagerServiceImpl.getGroupUserFromLogin(login);
+		
 		try {
-			File dir = new File(BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR));
-			if (dir.exists() && dir.isDirectory()) {
-				Set<String> res = new TreeSet<String>();
-				for (File file : dir.listFiles()) {
-					if (file.getName().endsWith(".properties") && !file.isDirectory()) {
-						res.add(file.getName().substring(0, file.getName().lastIndexOf('.')));
+			String root = new File(BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR)).getAbsolutePath();
+			for (String user : users) {
+				File dir = new File(root + "/" + user);
+				if (dir.exists()) {
+					for (File file : dir.listFiles()) {
+						if (file.getName().endsWith(".properties") && !file.isDirectory()) {
+							res.add(file.getName().substring(0, file.getName().lastIndexOf('.')));
+						}
 					}
+				} else {
+					dir.mkdir();
 				}
-				return res;
 			}
+			return res;
 			
 		} catch (BiomajException e) {
 			e.printStackTrace();
@@ -136,132 +159,229 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 	}
 
 	@Override
-	public Map<String, Map<String, String>> getIncludedProperties(String bankName, Collection<String> included) {
+	public Map<String, Map<String, String>> getIncludedProperties(String login, String bankName, Collection<String> included) {
 		Map<String, Map<String,String>> res = new HashMap<String, Map<String,String>>();
 		if (!bankName.equals("global")) {
-			res.put("global.properties", getFileProperties("global.properties", null));
-			Map<String, String> props = getFileProperties(bankName + ".properties", null);
+			res.put("global.properties", getGlobalPropertiesAsPrefixedMap());
+			Map<String, String> props = getFileProperties(login, bankName + ".properties", null);
 			if (props != null && props.containsKey("misc__include.properties")) {
 				String[] includedProps = props.get("misc__include.properties").split(",");
 				for (String file : includedProps) {
 					if (!file.trim().isEmpty()) {
-						res.put(file, getFileProperties(file, null));
+						String nameWithoutUser = file.substring(file.indexOf('/') + 1);
+						res.put(nameWithoutUser, getFileProperties(login, nameWithoutUser, null));
 					}
 				}
 			}
 			for (String name : included) {
 				if (name.equals(CREATE_NEW_INCLUDE)) {
 					String test = "include/include_file" + inc++ + ".properties";
-					while (propertiesExist(test))
+					while (propertiesExist(login, test))
 						test = "include/include_file" + inc++ + ".properties";
+//					res.put(test.substring(test.indexOf('/') + 1), new HashMap<String, String>());
 					res.put(test, new HashMap<String, String>());
-				} else if (!propertiesExist(name)) // New include file
+				} else if (!propertiesExist(login, name)) // New include file
 					res.put(name, new HashMap<String, String>());
 				else
-					res.put(name, getFileProperties(name, null));
+					res.put(name, getFileProperties(login, name, null));
 			}
 		}
 		return res;
 	}
 	
 	@Override
-	public void save(String type, String bankName, String newName, Map<String, String> newProps, boolean runUpdate) {
-		Properties props = new Properties();
+	public int save(String login, String type, String bankName, String newName, Map<String, String> newProps, boolean runUpdate) {
+		
+		Properties finalProps = new Properties();
+		
+		String userRoot = canEditProperties(login, newName);
+		if (userRoot == null) // Try another name
+			return 0;
+		
 		
 		if (type.equals(TYPE_LOCAL)) {
-			props = getBankOnlyProperties(bankName);
-			Map<String, String> tmp = getFileProperties(bankName + ".properties", null);
+			finalProps = getBankOnlyProperties(login, bankName);
+			Map<String, String> tmp = null;
+			if (UserManagerServiceImpl.adminLogin().equals(login) && bankName.endsWith("global"))
+				tmp = getGlobalPropertiesAsPrefixedMap();
+			else
+				tmp = getFileProperties(login, bankName + ".properties", null);
 			
 			/*
 			 * Remove previous processes.
 			 */
 			for (String key : tmp.keySet()) {
 				String[] split = key.split("__");
-				if (split[0].equals("misc") && !split[1].equals("include.properties"))
-					props.remove(split[1]);
+				if (split[0].equals("misc") && !split[1].equals("include.properties")
+						&& !split[1].equals("database.login")
+						&& !split[1].equals("database.password")
+						&& !split[1].equals("database.driver")
+						&& !split[1].equals("database.url")
+						&& !split[1].equals("database.type")) {
+					finalProps.remove(split[1]);
+				}
 			}
 		}
 		
-		for (String key : newProps.keySet()) {
-			String strippedKey = key.split(PROP_SEP)[1];
+		if (newProps != null) {
 			
-			/*
-			 * Handle PRE/REMOVE_PROCESS.db.pre.process special case.
-			 * Definition of pre and remove processes cannot be done via BLOCKS.
-			 */
-			if (strippedKey.equals("BLOCKS") && (newProps.get(key).equals("PRE_PROCESS") || newProps.get(key).equals("REMOVE_PROCESS")))
-				continue;
-			
-			if (strippedKey.contains("db.pre.process"))
-				strippedKey = "db.pre.process";
-			else if (strippedKey.contains("db.remove.process"))
-				strippedKey = "db.remove.process";
-			
-			String value = newProps.get(key);
-			if (value == null)
-				value = "";
-			
-			if (value.equals("__deleted"))
-				props.remove(strippedKey);
-			else {
-				props.put(strippedKey, value);
+			for (String key : newProps.keySet()) {
+				
+				String strippedKey = key.split(PROP_SEP)[1];
+				
+				String value = newProps.get(key);
+				if (value == null)
+					value = "";
+				
+				/*
+				 * Handle PRE/REMOVE_PROCESS.db.pre.process special case.
+				 * Definition of pre and remove processes cannot be done via BLOCKS.
+				 */
+				if (strippedKey.equals("BLOCKS") && (newProps.get(key).equals("PRE_PROCESS") || newProps.get(key).equals("REMOVE_PROCESS")))
+					continue;
+				
+				if (strippedKey.contains("db.pre.process"))
+					strippedKey = "db.pre.process";
+				else if (strippedKey.contains("db.remove.process"))
+					strippedKey = "db.remove.process";
+				else if (strippedKey.equals("include.properties") && !value.equals("__deleted")) {
+					String[] includes = value.split(",");
+					String newIncl = "";
+					for (String incl : includes) {
+						String path = canEditProperties(login, incl);
+						if (path != null) {
+							newIncl += path.substring(getConfDir().length() + 1) + "/" + incl + ",";
+						}
+					}
+					value = newIncl.substring(0, newIncl.length() - 1);
+				}
+				
+				if (value.equals("__deleted"))
+					finalProps.remove(strippedKey);
+				else {
+					finalProps.put(strippedKey, value);
+				}
 			}
 		}
 		
 		try {
-			String fileName;
-			if (type.equals("included"))
-				fileName = BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR) + "/include/" + newName + ".properties";
-			else
-				fileName = BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR) + "/" + newName + ".properties";
+			
+			String fileName = userRoot + "/" + newName + ".properties";
+			
 			PrintWriter pw = new PrintWriter(new File(fileName));
-			props.store(pw, "Generated by BmajWatcher");
+			finalProps.store(pw, "Generated by BmajWatcher");
 			pw.close();
 			
-			if (runUpdate)
+			if (runUpdate && !newName.equals("global"))
 				new BiomajLauncherServiceImpl().startUpdate(newName);
 			
-			
+			return 1;
 		} catch (FileNotFoundException e) {
-			e.printStackTrace();
-		} catch (BiomajException e) {
-			e.printStackTrace();
+			log.error(e);
 		} catch (IOException e) {
-			e.printStackTrace();
+			log.error(e);
 		}
+		
+		return -1;
 	}
 	
 	
-	
 	/**
 	 * Gets a bank's properties with no inheritance.
 	 * 
+	 * @param login identifies directories to look into
+	 * @param name file name
+	 * @param bankProperties don't remember
+	 * 
 	 * @return
 	 */
-	private Map<String, String> getFileProperties(String name, Properties bankProperties) {
+	private Map<String, String> getFileProperties(String login, String name, Properties bankProperties) {
+
 		Map<String, String> res = new HashMap<String, String>();
-		try {
-			File file = new File(BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR) + "/" + name);
-			if (file.exists()) {
-				Properties props = new Properties();
-				try {
-					props.load(new FileReader(file));
-				} catch (FileNotFoundException e) {
-					e.printStackTrace();
-				} catch (IOException e) {
-					e.printStackTrace();
-				}
-				for (Object o : props.keySet()) {
-					res.put(prefixProperty(o.toString(), bankProperties), props.getProperty(o.toString()));
-				}
-				return res;
+		File bank = getFileForUserGroup(login, name);
+		if (bank != null) {
+			Properties props = new Properties();
+			try {
+				FileReader fr = new FileReader(bank);
+				props.load(fr);
+				fr.close();
+			} catch (FileNotFoundException e) {
+				e.printStackTrace();
+			} catch (IOException e) {
+				e.printStackTrace();
+			}
+			for (Object o : props.keySet()) {
+				res.put(prefixProperty(o.toString(), bankProperties), props.getProperty(o.toString()));
 			}
-		} catch (BiomajException e) {
-			e.printStackTrace();
 		}
+		
 		return res;
 	}
 	
+	/**
+	 * If exists, returns file corresponding to given file name for the given
+	 * user.
+	 * 
+	 * @param login
+	 * @param fileName
+	 * @return null 
+	 */
+	private File getFileForUserGroup(String login, String fileName) {
+		List<String> users = null;
+		if (login.equals(UserManagerServiceImpl.adminLogin()))
+			users = UserManagerServiceImpl.getUserList();
+		else
+			users = UserManagerServiceImpl.getGroupUserFromLogin(login);
+		
+		String newName = fileName.endsWith(".properties") ? fileName : fileName + ".properties";
+		String root = getConfDir();
+		File bank = null;
+		for (String user : users) {
+			if ((bank = new File(root + "/" + user + "/" + newName)).exists())
+				return bank;
+		}
+		
+		return null;
+	}
+	
+	private Map<String, String> getGlobalPropertiesAsPrefixedMap() {
+		Map<String, String> res = new HashMap<String, String>();
+		
+		File global = new File(getConfDir() + "/global.properties");
+		if (global.exists()) {
+			Properties props = new Properties();
+			try {
+				FileReader fr = new FileReader(global);
+				props.load(fr);
+				fr.close();
+			} catch (FileNotFoundException e) {
+				e.printStackTrace();
+			} catch (IOException e) {
+				e.printStackTrace();
+			}
+			for (Object o : props.keySet()) {
+				res.put(prefixProperty(o.toString(), null), props.getProperty(o.toString()));
+			}
+		}
+		
+		return res;
+	}
+	
+	public static Properties getGlobalProperties() {
+		Properties global = new Properties();
+		try {
+			FileReader fr = new FileReader(getConfDir() + "/global.properties");
+			global.load(fr);
+			fr.close();
+		} catch (FileNotFoundException e) {
+			log.error(e);
+		} catch (IOException e) {
+			log.error(e);
+		}
+		
+		return global;
+	}
+	
 	/**
 	 * Prefixes a property by its category as defined in bankProperties.properties
 	 * and by its inheritance.
@@ -285,29 +405,36 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 	}
 	
 	
-	private Properties getBankOnlyProperties(String bankName) {
-		File file;
-		try {
-			file = new File(BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR) + "/" + bankName + ".properties");
-		} catch (BiomajException e1) {
-			e1.printStackTrace();
-			return null;
-		}
+	private Properties getBankOnlyProperties(String login, String bankName) {
+		String newBankName = bankName.endsWith(".properties") ? bankName : bankName + ".properties";
+		
+		if (newBankName.equals("global.properties") && login.equals(UserManagerServiceImpl.adminLogin()))
+			return getGlobalProperties();
+		
 		Properties bankOnlyProperties = new Properties();
-		try {
-			if (file.exists())
-				bankOnlyProperties.load(new FileReader(file));
-		} catch (FileNotFoundException e) {
-			e.printStackTrace();
-		} catch (IOException e) {
-			e.printStackTrace();
+		
+		String parent = canEditProperties(login, newBankName);
+		
+		if (parent != null) {
+			File file = new File(parent + "/" + newBankName);
+			try {
+				if (file.exists()) {
+					FileReader fr = new FileReader(file);
+					bankOnlyProperties.load(fr);
+					fr.close();
+				}
+			} catch (FileNotFoundException e) {
+				e.printStackTrace();
+			} catch (IOException e) {
+				e.printStackTrace();
+			}
 		}
 		
 		return bankOnlyProperties;
 	}
 	
 	/**
-	 * Modifies pre/post processing properties regarding these rules :
+	 * Modifies pre/post processing properties according to these rules :
 	 * - Pre-processes cannot be declared via BLOCKS (only db.pre.process)
 	 * - If BLOCKS property is present, any db.post.process property will not be taken
 	 * into account.
@@ -347,7 +474,8 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 			ref = 2;
 		
 		Map<String, String> emptyBank = new HashMap<String, String>();
-		Properties global = getBankOnlyProperties("global");
+		Properties global = getGlobalProperties();
+		
 		for (Object key : categorizedProps.keySet()) {
 			String keyString = key.toString();
 			int category = Integer.valueOf(categorizedProps.getProperty(keyString));
@@ -363,7 +491,7 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 		}
 		
 		Map<String, Map<String, String>> res = new HashMap<String, Map<String,String>>();
-		res.put("global.properties", getFileProperties("global.properties", null));
+		res.put("global.properties", getGlobalPropertiesAsPrefixedMap());
 		res.put(pattern + ".properties", emptyBank);
 		
 		return res;
@@ -377,7 +505,11 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 			Properties propertyHelp = new Properties();
 			propertyHelp.load(new FileReader(new File(serverPath + "/propertyHelp.properties")));
 			for (Object key : propertyHelp.keySet()) {
-				res.put(key.toString(), propertyHelp.getProperty(key.toString()));
+				String type = properties.getProperty(key.toString());
+				if (type.contains(">"))
+					type = type.substring(0, type.indexOf('>'));
+				
+				res.put(key.toString(), type + ":" + propertyHelp.getProperty(key.toString()));
 			}
 		} catch (FileNotFoundException e) {
 			e.printStackTrace();
@@ -413,17 +545,17 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 	 * @param included name of the included properties files
 	 * @return
 	 */
-	private Map<String, String> getBankProperties(String bankName, Collection<String> included) {
+	private Map<String, String> getBankProperties(String login, String bankName, Collection<String> included) {
 		Map<String, String> res = new HashMap<String, String>();
-		Properties bankProps = getBankOnlyProperties(bankName);
+		Properties bankProps = getBankOnlyProperties(login, bankName);
 		fixProcesses(bankProps);
-		Properties globalProps = getBankOnlyProperties("global");
+		Properties globalProps = getGlobalProperties();
 		fixProcesses(globalProps);
 		List<Properties> includedProps = new ArrayList<Properties>();
 		
 		for (String includedName : included) {
-			if (!includedName.equals(CREATE_NEW_INCLUDE) && propertiesExist(includedName)) {
-				Properties p = getBankOnlyProperties(includedName.substring(0, includedName.lastIndexOf('.')));
+			if (!includedName.equals(CREATE_NEW_INCLUDE) && propertiesExist(login, includedName)) {
+				Properties p = getBankOnlyProperties(login, includedName);
 				fixProcesses(p);
 				includedProps.add(p);
 			}
@@ -432,7 +564,7 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 			String[] includedFiles = bankProps.getProperty("include.properties").split(",");
 			for (String file : includedFiles) {
 				if (!file.trim().isEmpty() && !included.contains(file)) {
-					Properties p = getBankOnlyProperties(file.substring(0, file.lastIndexOf('.')));
+					Properties p = getBankOnlyProperties(login, file);
 					fixProcesses(p);
 					includedProps.add(p);
 				}
@@ -497,19 +629,67 @@ public class BankEditingServiceImpl extends RemoteServiceServlet implements Bank
 	}
 	
 	@Override
-	public boolean propertiesFileExists(String fileName) {
-		return propertiesExist(fileName);
+	public boolean propertiesFileExists(String login, String fileName) {
+		return propertiesExist(login, fileName);
 	}
 	
-	private boolean propertiesExist(String fileName) {
-		try {
-			File file = new File(BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR) + "/" + fileName);
-			return file.exists();
-		} catch (BiomajException e) {
-			log.error(e);
-			return false;
+	private boolean propertiesExist(String login, String fileName) {
+		return getFileForUserGroup(login, fileName) != null;
+	}
+
+	/**
+	 * Returns parent dir path if given file can be created or overriden
+	 * by given user, i.e. file with same name doesn't exist
+	 * for another group.
+	 * 
+	 * @param login
+	 * @param fileName
+	 * @return
+	 */
+	private String canEditProperties(String login, String fileName) {
+		
+		String newName = fileName.endsWith(".properties") ? fileName : fileName + ".properties";
+		
+		if (newName.equals("global.properties")) {
+			return getConfDir();
 		}
 		
+		File bank = null;
+		String root = getConfDir();
+		File dir = new File(root);
+		if (dir.exists() && dir.isDirectory()) {
+			for (File file : dir.listFiles()) {
+				if (file.isDirectory() && new File(file.getAbsolutePath() + "/" + newName).exists()) {
+					bank = file;
+					break;
+				}
+			}
+		}
+		
+		if (bank == null) // File was not found, can create under login
+			return root + "/" + login;
+		else if (login.equals(UserManagerServiceImpl.adminLogin()))
+			return bank.getAbsolutePath();
+		
+		List<String> usersOfGroup = UserManagerServiceImpl.getGroupUserFromLogin(login);
+		String path = bank.getAbsolutePath();
+
+		for (String user : usersOfGroup) {
+			if (path.endsWith("/" + user)) // File exist within user group, Ok to edit
+				return path;
+		}
+		
+		// File exist for another group, cannot create it
+		return null;
+	}
+	
+	private static String getConfDir() {
+		try {
+			return BiomajInformation.getInstance().getProperty(BiomajInformation.WORKFLOWSDIR);
+		} catch (BiomajException ex) {
+			log.error(ex);
+			return null;
+		}
 	}
 
 }
